/*
 * $Id: dumper.cpp,v 1.19 2016/04/29 13:05:42 rp Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * PLOP pCOS sample application for dumping PDF information
 */

#include <iostream>
#include <sstream>
#include <string>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

namespace
{
const wstring	docoptlist(L"requiredmode=minimum");
const wstring	globaloptlist(L"");

wstring get_wstring(const PLOP& plop, const string& utf8_string);
}

int
main(int argc, char **argv)
{
    PLOP	plop;

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    if (argc != 2)
    {   
        wcerr << L"usage: dumper <filename>" << endl;
        return(2);
    }

    try
    {
	int		count, pcosmode, recipientcount, plainmetadata;
	int		objtype, i, doc;
	wstring 	optlist;

	plop.set_option(globaloptlist);

	optlist = L"searchpath={" + searchpath + L"}";
	plop.set_option(optlist);

        /*
         * Caution: For simplicity we assume that the program arguments are
         * encoded as UTF-8, which might not be true in all cases!
         */
        const wstring doc_name(get_wstring(plop, string(argv[1])));
	if ((doc = plop.open_document(doc_name, docoptlist)) == -1)
	{
	    wcout << L"ERROR: " << plop.get_errmsg() << endl;
	    return 98;
	}

	/* --------- general information (always available) */

	pcosmode = static_cast<int>(plop.pcos_get_number(doc, L"pcosmode"));

	wcout << L"       File name: " << plop.pcos_get_string(doc, L"filename")
		<< endl;

	wcout << L"     PDF version: " <<
	    plop.pcos_get_string(doc, L"pdfversionstring") << endl;

	wcout << L"      Encryption: " <<
	    plop.pcos_get_string(doc, L"encrypt/description") << endl;

	recipientcount =
		static_cast<int>(plop.pcos_get_number( doc, L"length:encrypt/recipients"));
	if (recipientcount > 0)
	{
	    wcout << L"Recipient groups: " << recipientcount << endl;
	    wcout << L"      Digital ID: " << endl;

	    if (pcosmode == 2)
		wcout << L"supplied digital ID has master permissions" << endl;
	    else if (pcosmode == 1)
	        wcout << L"supplied digital ID doesn't have master permissions" << endl;
	    else
	        wcout << L"no suitable digital ID supplied" << endl;
	}

	wcout << L"       Master pw: " <<
	    (plop.pcos_get_number(doc, L"encrypt/master") ? L"yes" : L"no") 
	    	<< endl;

	wcout << L"         User pw: " <<
	    (plop.pcos_get_number(doc, L"encrypt/user") ? L"yes" : L"no") << endl;

	wcout << L"    Text copying: " <<
	    (plop.pcos_get_number(doc, L"encrypt/nocopy") ? L"no" : L"yes")
		<< endl;

	wcout << L"      Linearized: " <<
	    (plop.pcos_get_number(doc, L"linearized") ? L"yes" : L"no") << endl;

	if (pcosmode == 0)
	{
	    wcerr << L"    Minimum mode: no more information available\n\n" << endl;
	    plop.close_document(doc, L"");
	    return 0;
	}

	/* --------- more details (requires at least user password) */

	wcout << L"    PDF/X status: " << plop.pcos_get_string(doc, L"pdfx") << endl;

	wcout << L"    PDF/A status: " << plop.pcos_get_string(doc, L"pdfa") << endl;
	wcout << L"   PDF/UA status: " << plop.pcos_get_string(doc, L"pdfua") << endl;
	wcout << L"   PDF/VT status: " << plop.pcos_get_string(doc, L"pdfvt") << endl;

	wcout << L"        XFA data: "
	        << (static_cast<int>(
                        plop.pcos_get_number(doc, L"type:/Root/AcroForm/XFA"))
	                                            != pcos_ot_null
	                                    ? L"yes" : L"no") << endl;

	wcout << L"      Tagged PDF: " <<
	    (plop.pcos_get_number(doc, L"tagged") ? L"yes" : L"no") << endl
	    << endl;

        /* --------- analyze digital signatures */

	count = (int) plop.pcos_get_number(doc, L"length:signaturefields");

        for (i=0; i < count; i++)
        {
            wstring sigtype;
	    wostringstream sigfields;

	    sigfields << L"signaturefields[" << i << L"]/fullname";
            wcout << L"   Sig. field #" << i+1 << ": '"
                    << plop.pcos_get_string(doc, sigfields.str()) << L"',";

            sigfields.str(L"");
	    sigfields << L"signaturefields[" << i << L"]/visible";
            if (plop.pcos_get_number(doc, sigfields.str()))
                wcout << L" visible";
            else
                wcout << L" invisible";

            sigfields.str(L"");
	    sigfields << L"signaturefields[" << i << L"]/sigtype";
            sigtype = plop.pcos_get_string(doc, sigfields.str());

            if (sigtype == L"none")
            {
                wcout << L", unsigned\n";
                continue;
            }

            wcout << " " << sigtype << " signature";

            if (sigtype == L"certification")
            {
                sigfields.str(L"");
                sigfields << L"signaturefields[" << i << L"]/permissions";
                wcout << L" (" << plop.pcos_get_string(doc,
                                          sigfields.str()) << L" allowed)";
            }

            sigfields.str(L"");
            sigfields << L"signaturefields[" << i << L"]/cades";
            if (plop.pcos_get_number(doc, sigfields.str()))
                wcout << L", CAdES";

            wcout << endl;
        }
        if (count > 0)
        {
            printf("No. of revisions: %d\n\n",
                    static_cast<int>(plop.pcos_get_number(doc, L"revisions")));
        }

        wcout << L"  Reader-enabled: "
                << plop.pcos_get_string(doc, L"usagerights") << endl;


	wcout << L"    No. of pages: " <<
	    static_cast<int>(plop.pcos_get_number(doc, L"length:pages")) << endl;

	wcout << L"     Page 1 size: width="
	     << plop.pcos_get_number(doc, L"pages[0]/width")
	     << L", height="
	     << plop.pcos_get_number(doc, L"pages[0]/height") << endl;

	count = static_cast<int>(plop.pcos_get_number(doc, L"length:fonts"));
	wcout << L"    No. of fonts: " << count << endl;

	for (i=0; i < count; i++)
	{
	    wostringstream fonts;

	    fonts << L"fonts[" << i << L"]/embedded";
	    if (plop.pcos_get_number(doc, fonts.str()))
		wcout << L"embedded ";
	    else
		wcout << L"unembedded ";

	    fonts.str(L"");
	    fonts << L"fonts[" << i << L"]/type";
	    wcout << plop.pcos_get_string(doc, fonts.str()) << L" font ",

	    fonts.str(L"");
	    fonts << L"fonts[" << i << L"]/name";
	    wcout << plop.pcos_get_string(doc, fonts.str()) << endl;
	}

	wcout << endl;

	plainmetadata =
		static_cast<int>(plop.pcos_get_number(doc, L"encrypt/plainmetadata"));

	if (pcosmode == 1 && !plainmetadata &&
	    (static_cast<int>(plop.pcos_get_number(doc, L"encrypt/nocopy"))))
	{
	    wcout << L"Restricted mode: no more information available"
		 << endl << endl;
	    plop.close_document(doc, L"");
	    return 0;
	}

	/* --------- document info keys and XMP metadata (requires master pw) */

	count = (int) plop.pcos_get_number(doc, L"length:/Info");

	for (i = 0; i < count; i++)
	{
	    wostringstream info;

	    info << L"type:/Info[" << i << L"]";
	    objtype = static_cast<int>(plop.pcos_get_number(doc, info.str()));

	    info.str(L"");
	    info << L"/Info[" << i << L"].key";
	    wcout.width(16);
	    wcout << plop.pcos_get_string(doc, info.str()) << L": ";

	    /* Info entries can be stored as wstring or name objects */
	    if (objtype == pcos_ot_string || objtype == pcos_ot_name)
	    {
		info.str(L"");
		info << L"/Info[" << i << L"]";
		wcout << L"'" << plop.pcos_get_string(doc, info.str()) << L"'" 
			<< endl;
	    }
	    else
	    {
		info.str(L"");
		info << L"type:/Info[" << i << L"]";
		wcout << L"(" << plop.pcos_get_string(doc, info.str())
		       	<< L" object)" << endl;
	    }
	}

	wcout << endl << L"   XMP meta data: ";

	objtype = static_cast<int>(plop.pcos_get_number(doc, L"type:/Root/Metadata"));
	if (objtype == pcos_ot_stream)
	{
	    const unsigned char *contents;
	    int len;

	    contents = plop.pcos_get_stream(doc, &len, L"", L"/Root/Metadata");
	    wcout << len << L" bytes ";

	    /*
	     * This demonstrates Unicode conversion.
	     */
	    const string utf32 = plop.convert_to_unicode(L"utf8",
                string(reinterpret_cast<const char *>(contents), static_cast<size_t>(len)),
                L"outputformat=utf32");
	    wcout << L"(" << utf32.size() / 4 << L" Unicode characters)";
	    wcout << endl << endl;
	}
	else
	{
	    wcout << L"not present" << endl << endl;
	}

	plop.close_document(doc, L"");
    }
    catch (PLOP::Exception &ex) {
        wcerr << L"PLOP exception occurred in dumper:" << endl;
	wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
	            << L": " << ex.get_errmsg() << endl;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given string, assuming the string is encoded in
 * UTF-8.
 */
wstring get_wstring(const PLOP& plop, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf16");
        break;

    case 4:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *>(wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace

