/* $Id: SignServlet.java,v 1.8 2016/03/24 13:27:12 rp Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * sample for creating a signed document
 */

import javax.servlet.http.HttpServlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import com.pdflib.PLOPException;
import com.pdflib.plop;

public class SignServlet extends HttpServlet {
    private static final long serialVersionUID = -4446230350807159305L;

    public void service(ServletRequest request, ServletResponse response)
            throws ServletException {
        plop plop = null;

        /* Basic signature options */
        String sign_opts =
            "ltv=try " +         /* use ltv=full to ensure LTV status */
            /* for use with PKCS11
            "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
            */
	    "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} ";        
        
        try {
            /* parameters for the input document */
            final String in_filename = "invoice.pdf";
            /* parameters for the input document */
            String optlist;
            String fieldoptlist = "";
            
            /* Set up searchpath for the required data. */
            final ServletContext context = getServletContext();
            final String searchpath = context.getRealPath("/WEB-INF/data");

            int doc;
            int visdoc = -1;
            boolean visualize = false;
            String visfile = "signing_man_pdfa1a.pdf";

            plop = new plop();

            /* open unprotected input file without any password */
            optlist = "searchpath {" + searchpath + "} ";
            plop.set_option(optlist);
            if ((doc = plop.open_document(in_filename, "")) == -1) {
                throw new Exception("Error: " + plop.get_apiname() + ": "
                        + plop.get_errmsg());
            }
            optlist = sign_opts;

            if (visualize) {
                if ((visdoc = plop.open_document(visfile, "")) == -1) {
                    throw new Exception("Error: " + plop.get_apiname() + ": "
                            + plop.get_errmsg());
                }
                fieldoptlist = "field={visdoc=" + visdoc
                        + " rect={50 50 adapt adapt}} ";
                optlist = optlist + fieldoptlist;
            }

            if (plop.prepare_signature(optlist) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                                              + ": " + plop.get_errmsg());
            }

            if (plop.create_document("", "input=" + doc) == -1) {
                throw new Exception("Error: " + plop.get_apiname()
                                              + ": " + plop.get_errmsg());
            }

            final byte[] buf = plop.get_buffer();
            
            /* close input and output files */
            plop.close_document(doc, "");
            if (visualize) {
                plop.close_document(visdoc, "");
            }

            response.setContentType("application/pdf");
            response.setContentLength(buf.length);

            final ServletOutputStream out = response.getOutputStream();
            out.write(buf);
            out.close();
        }
        catch (PLOPException e) {
            throw new ServletException("PLOP exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            /* delete the PLOP object */
            if (plop != null)
                plop.delete();
        }
    }
}
